@@ -44,6 +44,7 @@
 extern "C" {
 #endif /* __cplusplus */
 
+#include "config_sample.h"
 #include "ithread.h"
 #include "ixml.h" /* for IXML_Document, IXML_Element */
 #include "upnp.h" /* for Upnp_EventType */
@@ -132,7 +133,7 @@ int SampleUtil_PrintEvent(
 	/*! [in] The type of callback event. */
 	Upnp_EventType EventType, 
 	/*! [in] The callback event structure. */
-	void *Event);
+	const void *Event);
 
 /*!
  * \brief This routine finds the first occurance of a service in a DOM
@@ -258,8 +259,10 @@ void linux_print(const char *format, ...)
 };
 #endif /* __cplusplus */
 
-#ifdef WIN32
-	#define snprintf	_snprintf
+#ifdef _WIN32
+	#if defined(_MSC_VER) && _MSC_VER < 1900
+		#define snprintf	_snprintf
+	#endif
 	#define strcasecmp	stricmp
 #endif
 
