@@ -165,7 +165,7 @@ void TvStateUpdate(
 
 void	TvCtrlPointHandleEvent(const char *, int, IXML_Document *); 
 void	TvCtrlPointHandleSubscribeUpdate(const char *, const Upnp_SID, int); 
-int		TvCtrlPointCallbackEventHandler(Upnp_EventType, void *, void *);
+int		TvCtrlPointCallbackEventHandler(Upnp_EventType, const void *, void *);
 
 /*!
  * \brief Checks the advertisement each device in the global device list.
@@ -182,7 +182,7 @@ void TvCtrlPointVerifyTimeouts(
 
 void	TvCtrlPointPrintCommands(void);
 void*	TvCtrlPointCommandLoop(void *);
-int		TvCtrlPointStart(print_string printFunctionPtr, state_update updateFunctionPtr, int combo);
+int		TvCtrlPointStart(char* iface, state_update updateFunctionPtr, int combo);
 int		TvCtrlPointStop(void);
 int		TvCtrlPointProcessCommand(char *cmdline);
 
