@@ -1,70 +1,72 @@
 /*******************************************************************************
  *
- * Copyright (c) 2000-2003 Intel Corporation 
- * All rights reserved. 
+ * Copyright (c) 2000-2003 Intel Corporation
+ * All rights reserved.
+ * Copyright (c) 2012 France Telecom All rights reserved.
  *
- * Redistribution and use in source and binary forms, with or without 
- * modification, are permitted provided that the following conditions are met: 
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions are met:
  *
- * - Redistributions of source code must retain the above copyright notice, 
- * this list of conditions and the following disclaimer. 
- * - Redistributions in binary form must reproduce the above copyright notice, 
- * this list of conditions and the following disclaimer in the documentation 
- * and/or other materials provided with the distribution. 
- * - Neither name of Intel Corporation nor the names of its contributors 
- * may be used to endorse or promote products derived from this software 
+ * - Redistributions of source code must retain the above copyright notice,
+ * this list of conditions and the following disclaimer.
+ * - Redistributions in binary form must reproduce the above copyright notice,
+ * this list of conditions and the following disclaimer in the documentation
+ * and/or other materials provided with the distribution.
+ * - Neither name of Intel Corporation nor the names of its contributors
+ * may be used to endorse or promote products derived from this software
  * without specific prior written permission.
- * 
- * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
- * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
- * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR 
- * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL INTEL OR 
- * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
- * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
- * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR 
- * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY 
+ *
+ * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
+ * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
+ * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
+ * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL INTEL OR
+ * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
+ * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
+ * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
+ * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
  * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
- * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
+ * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
  ******************************************************************************/
 
-
 #include "config.h"
 
-
 /*!
  * \file
  */
 
-
 #if EXCLUDE_GENA == 0
-#ifdef INCLUDE_CLIENT_APIS
+	#ifdef INCLUDE_CLIENT_APIS
 
+		#include "UpnpEventSubscribe.h"
+		#include "gena.h"
+		#include "httpparser.h"
+		#include "httpreadwrite.h"
+		#include "parsetools.h"
+		#include "statcodes.h"
+		#include "sysdep.h"
+		#include "upnpapi.h"
+		#include "uuid.h"
 
-#include "gena.h"
-#include "httpparser.h"
-#include "httpreadwrite.h"
-#include "parsetools.h"
-#include "statcodes.h"
-#include "sysdep.h"
-#include "uuid.h"
-#include "upnpapi.h"
-
+		#ifdef _WIN32
+			#if defined(_MSC_VER) && _MSC_VER < 1900
+				#define snprintf _snprintf
+			#endif
+		#endif
 
 extern ithread_mutex_t GlobalClientSubscribeMutex;
 
-
 /*!
  * \brief This is a thread function to send the renewal just before the
  * subscription times out.
  */
 static void GenaAutoRenewSubscription(
 	/*! [in] Thread data(upnp_timeout *) needed to send the renewal. */
-	IN void *input)
+	void *input)
 {
-	upnp_timeout *event = (upnp_timeout *) input;
-        struct Upnp_Event_Subscribe *sub_struct = (struct Upnp_Event_Subscribe *)event->Event;
+	upnp_timeout *event = (upnp_timeout *)input;
+	UpnpEventSubscribe *sub_struct = (UpnpEventSubscribe *)event->Event;
 	void *cookie;
 	Upnp_FunPtr callback_fun;
 	struct Handle_Info *handle_info;
@@ -72,26 +74,27 @@ static void GenaAutoRenewSubscription(
 	int eventType = 0;
 	int timeout = 0;
 	int errCode = 0;
-	UpnpString *tmpSID = UpnpString_new();
 
 	if (AUTO_RENEW_TIME == 0) {
-		UpnpPrintf( UPNP_INFO, GENA, __FILE__, __LINE__, "GENA SUB EXPIRED");
-		sub_struct->ErrCode = UPNP_E_SUCCESS;
+		UpnpPrintf(UPNP_INFO,
+			GENA,
+			__FILE__,
+			__LINE__,
+			"GENA SUB EXPIRED");
+		UpnpEventSubscribe_set_ErrCode(sub_struct, UPNP_E_SUCCESS);
 		send_callback = 1;
 		eventType = UPNP_EVENT_SUBSCRIPTION_EXPIRED;
 	} else {
-		UpnpPrintf(UPNP_INFO, GENA, __FILE__, __LINE__, "GENA AUTO RENEW");
-		timeout = sub_struct->TimeOut;
-		UpnpString_set_String(tmpSID, sub_struct->Sid);
-		errCode = genaRenewSubscription(
-			event->handle,
-			tmpSID,
+		UpnpPrintf(
+			UPNP_INFO, GENA, __FILE__, __LINE__, "GENA AUTO RENEW");
+		timeout = UpnpEventSubscribe_get_TimeOut(sub_struct);
+		errCode = genaRenewSubscription(event->handle,
+			UpnpEventSubscribe_get_SID(sub_struct),
 			&timeout);
-		sub_struct->ErrCode = errCode;
-		sub_struct->TimeOut = timeout;
-		if (errCode != UPNP_E_SUCCESS &&
-		    errCode != GENA_E_BAD_SID &&
-		    errCode != GENA_E_BAD_HANDLE) {
+		UpnpEventSubscribe_set_ErrCode(sub_struct, errCode);
+		UpnpEventSubscribe_set_TimeOut(sub_struct, timeout);
+		if (errCode != UPNP_E_SUCCESS && errCode != GENA_E_BAD_SID &&
+			errCode != GENA_E_BAD_HANDLE) {
 			send_callback = 1;
 			eventType = UPNP_EVENT_AUTORENEWAL_FAILED;
 		}
@@ -99,12 +102,13 @@ static void GenaAutoRenewSubscription(
 
 	if (send_callback) {
 		HandleReadLock();
-		if( GetHandleInfo( event->handle, &handle_info ) != HND_CLIENT ) {
+		if (GetHandleInfo(event->handle, &handle_info) != HND_CLIENT) {
 			HandleUnlock();
 			free_upnp_timeout(event);
 			goto end_function;
 		}
-		UpnpPrintf(UPNP_INFO, GENA, __FILE__, __LINE__, "HANDLE IS VALID");
+		UpnpPrintf(
+			UPNP_INFO, GENA, __FILE__, __LINE__, "HANDLE IS VALID");
 
 		/* make callback */
 		callback_fun = handle_info->Callback;
@@ -116,11 +120,9 @@ static void GenaAutoRenewSubscription(
 	free_upnp_timeout(event);
 
 end_function:
-	UpnpString_delete(tmpSID);
 	return;
 }
 
-
 /*!
  * \brief Schedules a job to renew the subscription just before time out.
  *
@@ -129,58 +131,60 @@ end_function:
  */
 static int ScheduleGenaAutoRenew(
 	/*! [in] Handle that also contains the subscription list. */
-	IN int client_handle,
+	int client_handle,
 	/*! [in] The time out value of the subscription. */
-	IN int TimeOut,
+	int TimeOut,
 	/*! [in] Subscription being renewed. */
-	IN ClientSubscription *sub)
+	GenlibClientSubscription *sub)
 {
-	struct Upnp_Event_Subscribe *RenewEventStruct = NULL;
+	UpnpEventSubscribe *RenewEventStruct = NULL;
 	upnp_timeout *RenewEvent = NULL;
 	int return_code = GENA_SUCCESS;
 	ThreadPoolJob job;
-	const UpnpString *tmpSID = UpnpClientSubscription_get_SID(sub);
-	const UpnpString *tmpEventURL = UpnpClientSubscription_get_EventURL(sub);
+
+	memset(&job, 0, sizeof(job));
 
 	if (TimeOut == UPNP_INFINITE) {
 		return_code = GENA_SUCCESS;
 		goto end_function;
 	}
 
-	RenewEventStruct = (struct Upnp_Event_Subscribe *)malloc(sizeof (struct Upnp_Event_Subscribe));
+	RenewEventStruct = UpnpEventSubscribe_new();
 	if (RenewEventStruct == NULL) {
 		return_code = UPNP_E_OUTOF_MEMORY;
 		goto end_function;
 	}
 
-	RenewEvent = (upnp_timeout *) malloc(sizeof(upnp_timeout));
+	RenewEvent = (upnp_timeout *)malloc(sizeof(upnp_timeout));
 	if (RenewEvent == NULL) {
 		free(RenewEventStruct);
 		return_code = UPNP_E_OUTOF_MEMORY;
 		goto end_function;
 	}
+	memset(RenewEvent, 0, sizeof(upnp_timeout));
 
 	/* schedule expire event */
-	RenewEventStruct->ErrCode = UPNP_E_SUCCESS;
-	RenewEventStruct->TimeOut = TimeOut;
-	strcpy(RenewEventStruct->Sid, UpnpString_get_String(tmpSID));
-	strncpy(RenewEventStruct->PublisherUrl,
-		UpnpString_get_String(tmpEventURL), NAME_SIZE - 1);
+	UpnpEventSubscribe_set_ErrCode(RenewEventStruct, UPNP_E_SUCCESS);
+	UpnpEventSubscribe_set_TimeOut(RenewEventStruct, TimeOut);
+	UpnpEventSubscribe_set_SID(
+		RenewEventStruct, GenlibClientSubscription_get_SID(sub));
+	UpnpEventSubscribe_set_PublisherUrl(
+		RenewEventStruct, GenlibClientSubscription_get_EventURL(sub));
 
 	/* RenewEvent->EventType=UPNP_EVENT_SUBSCRIPTION_EXPIRE; */
 	RenewEvent->handle = client_handle;
 	RenewEvent->Event = RenewEventStruct;
 
-	TPJobInit(&job, (start_routine) GenaAutoRenewSubscription, RenewEvent);
+	TPJobInit(&job, (start_routine)GenaAutoRenewSubscription, RenewEvent);
 	TPJobSetFreeFunction(&job, (free_routine)free_upnp_timeout);
 	TPJobSetPriority(&job, MED_PRIORITY);
 
 	/* Schedule the job */
-	return_code = TimerThreadSchedule(
-		&gTimerThread,
+	return_code = TimerThreadSchedule(&gTimerThread,
 		TimeOut - AUTO_RENEW_TIME,
 		REL_SEC,
-		&job, SHORT_TERM,
+		&job,
+		SHORT_TERM,
 		&(RenewEvent->eventId));
 	if (return_code != UPNP_E_SUCCESS) {
 		free(RenewEvent);
@@ -188,15 +192,15 @@ static int ScheduleGenaAutoRenew(
 		goto end_function;
 	}
 
-	UpnpClientSubscription_set_RenewEventId(sub, RenewEvent->eventId);
+	GenlibClientSubscription_set_RenewEventId(sub, RenewEvent->eventId);
 
 	return_code = GENA_SUCCESS;
 
 end_function:
+
 	return return_code;
 }
 
-
 /*!
  * \brief Sends the UNSUBCRIBE gena request and recieves the response from the
  * 	device and returns it as a parameter.
@@ -205,19 +209,18 @@ end_function:
  */
 static int gena_unsubscribe(
 	/*! [in] Event URL of the service. */
-	IN const UpnpString *url,
+	const UpnpString *url,
 	/*! [in] The subcription ID. */
-	IN const UpnpString *sid,
+	const UpnpString *sid,
 	/*! [out] The UNSUBCRIBE response from the device. */
-	OUT http_parser_t *response )
+	http_parser_t *response)
 {
 	int return_code;
 	uri_type dest_url;
 	membuffer request;
 
 	/* parse url */
-	return_code = http_FixStrUrl(
-		UpnpString_get_String(url),
+	return_code = http_FixStrUrl(UpnpString_get_String(url),
 		UpnpString_get_Length(url),
 		&dest_url);
 	if (return_code != 0) {
@@ -227,11 +230,16 @@ static int gena_unsubscribe(
 	/* make request msg */
 	membuffer_init(&request);
 	request.size_inc = 30;
-	return_code = http_MakeMessage(
-		&request, 1, 1,
-		"q" "ssc" "Uc",
-		HTTPMETHOD_UNSUBSCRIBE, &dest_url,
-		"SID: ", UpnpString_get_String(sid));
+	return_code = http_MakeMessage(&request,
+		1,
+		1,
+		"q"
+		"ssc"
+		"Uc",
+		HTTPMETHOD_UNSUBSCRIBE,
+		&dest_url,
+		"SID: ",
+		UpnpString_get_String(sid));
 
 	/* Not able to make the message so destroy the existing buffer */
 	if (return_code != 0) {
@@ -241,9 +249,12 @@ static int gena_unsubscribe(
 	}
 
 	/* send request and get reply */
-	return_code = http_RequestAndResponse(
-		&dest_url, request.buf, request.length,
-		HTTPMETHOD_UNSUBSCRIBE, HTTP_DEFAULT_TIMEOUT, response);
+	return_code = http_RequestAndResponse(&dest_url,
+		request.buf,
+		request.length,
+		HTTPMETHOD_UNSUBSCRIBE,
+		HTTP_DEFAULT_TIMEOUT,
+		response);
 	membuffer_destroy(&request);
 	if (return_code != 0) {
 		httpmsg_destroy(&response->msg);
@@ -257,7 +268,6 @@ static int gena_unsubscribe(
 	return return_code;
 }
 
-
 /*!
  * \brief Subscribes or renew subscription.
  *
@@ -265,14 +275,14 @@ static int gena_unsubscribe(
  */
 static int gena_subscribe(
 	/*! [in] URL of service to subscribe. */
-	IN const UpnpString *url,
+	const UpnpString *url,
 	/*! [in,out] Subscription time desired (in secs). */
-	INOUT int *timeout,
+	int *timeout,
 	/*! [in] for renewal, this contains a currently held subscription SID.
 	 * For first time subscription, this must be NULL. */
-	IN const UpnpString *renewal_sid,
+	const UpnpString *renewal_sid,
 	/*! [out] SID returned by the subscription or renew msg. */
-	OUT UpnpString *sid)
+	UpnpString *sid)
 {
 	int return_code;
 	int parse_ret = 0;
@@ -283,6 +293,7 @@ static int gena_subscribe(
 	membuffer request;
 	uri_type dest_url;
 	http_parser_t response;
+	int rc = 0;
 
 	UpnpString_clear(sid);
 
@@ -291,16 +302,21 @@ static int gena_subscribe(
 		timeout = &local_timeout;
 	}
 	if (*timeout < 0) {
-		strcpy(timeout_str, "infinite");
-	} else if(*timeout < CP_MINIMUM_SUBSCRIPTION_TIME) {
-		sprintf(timeout_str, "%d", CP_MINIMUM_SUBSCRIPTION_TIME);
+		memset(timeout_str, 0, sizeof(timeout_str));
+		strncpy(timeout_str, "infinite", sizeof(timeout_str) - 1);
+	} else if (*timeout < CP_MINIMUM_SUBSCRIPTION_TIME) {
+		rc = snprintf(timeout_str,
+			sizeof(timeout_str),
+			"%d",
+			CP_MINIMUM_SUBSCRIPTION_TIME);
 	} else {
-		sprintf(timeout_str, "%d", *timeout);
+		rc = snprintf(timeout_str, sizeof(timeout_str), "%d", *timeout);
 	}
+	if (rc < 0 || (unsigned int)rc >= sizeof(timeout_str))
+		return UPNP_E_OUTOF_MEMORY;
 
 	/* parse url */
-	return_code = http_FixStrUrl(
-		UpnpString_get_String(url),
+	return_code = http_FixStrUrl(UpnpString_get_String(url),
 		UpnpString_get_Length(url),
 		&dest_url);
 	if (return_code != 0) {
@@ -312,34 +328,65 @@ static int gena_subscribe(
 	request.size_inc = 30;
 	if (renewal_sid) {
 		/* renew subscription */
-		return_code = http_MakeMessage(
-			&request, 1, 1,
-			"q" "ssc" "sscc",
-			HTTPMETHOD_SUBSCRIBE, &dest_url,
-			"SID: ", UpnpString_get_String(renewal_sid),
-			"TIMEOUT: Second-", timeout_str );
+		return_code = http_MakeMessage(&request,
+			1,
+			1,
+			"q"
+			"ssc"
+			"sscc",
+			HTTPMETHOD_SUBSCRIBE,
+			&dest_url,
+			"SID: ",
+			UpnpString_get_String(renewal_sid),
+			"TIMEOUT: Second-",
+			timeout_str);
 	} else {
 		/* subscribe */
 		if (dest_url.hostport.IPaddress.ss_family == AF_INET6) {
-			struct sockaddr_in6* DestAddr6 = (struct sockaddr_in6*)&dest_url.hostport.IPaddress;
-			return_code = http_MakeMessage(
-				&request, 1, 1,
-				"q" "sssdsc" "sc" "sscc",
-				HTTPMETHOD_SUBSCRIBE, &dest_url,
+			struct sockaddr_in6 *DestAddr6 =
+				(struct sockaddr_in6 *)&dest_url.hostport
+					.IPaddress;
+			return_code = http_MakeMessage(&request,
+				1,
+				1,
+				"q"
+				"sssdsc"
+				"sc"
+				"sscc",
+				HTTPMETHOD_SUBSCRIBE,
+				&dest_url,
 				"CALLBACK: <http://[",
-				(IN6_IS_ADDR_LINKLOCAL(&DestAddr6->sin6_addr) || strlen(gIF_IPV6_ULA_GUA) == 0) ?
-					gIF_IPV6 : gIF_IPV6_ULA_GUA,
-				"]:", LOCAL_PORT_V6, "/>",
+				(IN6_IS_ADDR_LINKLOCAL(&DestAddr6->sin6_addr) ||
+					strlen(gIF_IPV6_ULA_GUA) == 0)
+					? gIF_IPV6
+					: gIF_IPV6_ULA_GUA,
+				"]:",
+				(IN6_IS_ADDR_LINKLOCAL(&DestAddr6->sin6_addr) ||
+					strlen(gIF_IPV6_ULA_GUA) == 0)
+					? LOCAL_PORT_V6
+					: LOCAL_PORT_V6_ULA_GUA,
+				"/>",
 				"NT: upnp:event",
-				"TIMEOUT: Second-", timeout_str);
+				"TIMEOUT: Second-",
+				timeout_str);
 		} else {
-			return_code = http_MakeMessage(
-				&request, 1, 1,
-				"q" "sssdsc" "sc" "sscc",
-				HTTPMETHOD_SUBSCRIBE, &dest_url,
-				"CALLBACK: <http://", gIF_IPV4, ":", LOCAL_PORT_V4, "/>",
+			return_code = http_MakeMessage(&request,
+				1,
+				1,
+				"q"
+				"sssdsc"
+				"sc"
+				"sscc",
+				HTTPMETHOD_SUBSCRIBE,
+				&dest_url,
+				"CALLBACK: <http://",
+				gIF_IPV4,
+				":",
+				LOCAL_PORT_V4,
+				"/>",
 				"NT: upnp:event",
-				"TIMEOUT: Second-", timeout_str);
+				"TIMEOUT: Second-",
+				timeout_str);
 		}
 	}
 	if (return_code != 0) {
@@ -347,7 +394,8 @@ static int gena_subscribe(
 	}
 
 	/* send request and get reply */
-	return_code = http_RequestAndResponse(&dest_url, request.buf,
+	return_code = http_RequestAndResponse(&dest_url,
+		request.buf,
 		request.length,
 		HTTPMETHOD_SUBSCRIBE,
 		HTTP_DEFAULT_TIMEOUT,
@@ -367,22 +415,24 @@ static int gena_subscribe(
 
 	/* get SID and TIMEOUT */
 	if (httpmsg_find_hdr(&response.msg, HDR_SID, &sid_hdr) == NULL ||
-	    sid_hdr.length == 0 ||
-	    httpmsg_find_hdr( &response.msg, HDR_TIMEOUT, &timeout_hdr ) == NULL ||
-	    timeout_hdr.length == 0 ) {
-		httpmsg_destroy( &response.msg );
+		sid_hdr.length == 0 ||
+		httpmsg_find_hdr(&response.msg, HDR_TIMEOUT, &timeout_hdr) ==
+			NULL ||
+		timeout_hdr.length == 0) {
+		httpmsg_destroy(&response.msg);
 
 		return UPNP_E_BAD_RESPONSE;
 	}
 
 	/* save timeout */
-	parse_ret = matchstr(timeout_hdr.buf, timeout_hdr.length, "%iSecond-%d%0", timeout);
+	parse_ret = matchstr(
+		timeout_hdr.buf, timeout_hdr.length, "%iSecond-%d%0", timeout);
 	if (parse_ret == PARSE_OK) {
 		/* nothing to do */
 	} else if (memptr_cmp_nocase(&timeout_hdr, "Second-infinite") == 0) {
 		*timeout = -1;
 	} else {
-		httpmsg_destroy( &response.msg );
+		httpmsg_destroy(&response.msg);
 
 		return UPNP_E_BAD_RESPONSE;
 	}
@@ -399,15 +449,14 @@ static int gena_subscribe(
 	return UPNP_E_SUCCESS;
 }
 
-
 int genaUnregisterClient(UpnpClient_Handle client_handle)
 {
-	ClientSubscription *sub_copy = UpnpClientSubscription_new();
+	GenlibClientSubscription *sub_copy = GenlibClientSubscription_new();
 	int return_code = UPNP_E_SUCCESS;
 	struct Handle_Info *handle_info = NULL;
 	http_parser_t response;
 
-	while (TRUE) {
+	while (1) {
 		HandleLock();
 
 		if (GetHandleInfo(client_handle, &handle_info) != HND_CLIENT) {
@@ -419,16 +468,16 @@ int genaUnregisterClient(UpnpClient_Handle client_handle)
 			return_code = UPNP_E_SUCCESS;
 			break;
 		}
-		UpnpClientSubscription_assign(sub_copy, handle_info->ClientSubList);
-		RemoveClientSubClientSID(
-			&handle_info->ClientSubList,
-			UpnpClientSubscription_get_SID(sub_copy));
+		GenlibClientSubscription_assign(
+			sub_copy, handle_info->ClientSubList);
+		RemoveClientSubClientSID(&handle_info->ClientSubList,
+			GenlibClientSubscription_get_SID(sub_copy));
 
 		HandleUnlock();
 
 		return_code = gena_unsubscribe(
-			UpnpClientSubscription_get_EventURL(sub_copy),
-			UpnpClientSubscription_get_ActualSID(sub_copy),
+			GenlibClientSubscription_get_EventURL(sub_copy),
+			GenlibClientSubscription_get_ActualSID(sub_copy),
 			&response);
 		if (return_code == 0) {
 			httpmsg_destroy(&response.msg);
@@ -440,20 +489,17 @@ int genaUnregisterClient(UpnpClient_Handle client_handle)
 	HandleUnlock();
 
 exit_function:
-	UpnpClientSubscription_delete(sub_copy);
+	GenlibClientSubscription_delete(sub_copy);
 	return return_code;
 }
 
-
-#ifdef INCLUDE_CLIENT_APIS
-int genaUnSubscribe(
-	UpnpClient_Handle client_handle,
-	const UpnpString *in_sid)
+		#ifdef INCLUDE_CLIENT_APIS
+int genaUnSubscribe(UpnpClient_Handle client_handle, const UpnpString *in_sid)
 {
-	ClientSubscription *sub = NULL;
+	GenlibClientSubscription *sub = NULL;
 	int return_code = GENA_SUCCESS;
 	struct Handle_Info *handle_info;
-	ClientSubscription *sub_copy = UpnpClientSubscription_new();
+	GenlibClientSubscription *sub_copy = GenlibClientSubscription_new();
 	http_parser_t response;
 
 	/* validate handle and sid */
@@ -469,12 +515,12 @@ int genaUnSubscribe(
 		return_code = GENA_E_BAD_SID;
 		goto exit_function;
 	}
-	UpnpClientSubscription_assign(sub_copy, sub);
+	GenlibClientSubscription_assign(sub_copy, sub);
 	HandleUnlock();
 
 	return_code = gena_unsubscribe(
-		UpnpClientSubscription_get_EventURL(sub_copy),
-		UpnpClientSubscription_get_ActualSID(sub_copy),
+		GenlibClientSubscription_get_EventURL(sub_copy),
+		GenlibClientSubscription_get_ActualSID(sub_copy),
 		&response);
 	if (return_code == 0) {
 		httpmsg_destroy(&response.msg);
@@ -491,27 +537,30 @@ int genaUnSubscribe(
 	HandleUnlock();
 
 exit_function:
-	UpnpClientSubscription_delete(sub_copy);
+	GenlibClientSubscription_delete(sub_copy);
 	return return_code;
 }
-#endif /* INCLUDE_CLIENT_APIS */
+		#endif /* INCLUDE_CLIENT_APIS */
 
-
-#ifdef INCLUDE_CLIENT_APIS
-int genaSubscribe(
-	UpnpClient_Handle client_handle,
+		#ifdef INCLUDE_CLIENT_APIS
+int genaSubscribe(UpnpClient_Handle client_handle,
 	const UpnpString *PublisherURL,
 	int *TimeOut,
 	UpnpString *out_sid)
 {
 	int return_code = GENA_SUCCESS;
-	ClientSubscription *newSubscription = UpnpClientSubscription_new();
+	GenlibClientSubscription *newSubscription =
+		GenlibClientSubscription_new();
 	uuid_upnp uid;
 	Upnp_SID temp_sid;
 	Upnp_SID temp_sid2;
 	UpnpString *ActualSID = UpnpString_new();
 	UpnpString *EventURL = UpnpString_new();
 	struct Handle_Info *handle_info;
+	int rc = 0;
+
+	memset(temp_sid, 0, sizeof(temp_sid));
+	memset(temp_sid2, 0, sizeof(temp_sid2));
 
 	UpnpPrintf(UPNP_INFO, GENA, __FILE__, __LINE__, "GENA SUBSCRIBE BEGIN");
 
@@ -520,9 +569,9 @@ int genaSubscribe(
 	HandleReadLock();
 	/* validate handle */
 	if (GetHandleInfo(client_handle, &handle_info) != HND_CLIENT) {
-		HandleUnlock();
-
-		return GENA_E_BAD_HANDLE;
+		return_code = GENA_E_BAD_HANDLE;
+		SubscribeLock();
+		goto error_handler;
 	}
 	HandleUnlock();
 
@@ -531,21 +580,29 @@ int genaSubscribe(
 	return_code = gena_subscribe(PublisherURL, TimeOut, NULL, ActualSID);
 	HandleLock();
 	if (return_code != UPNP_E_SUCCESS) {
-		UpnpPrintf( UPNP_CRITICAL, GENA, __FILE__, __LINE__,
-			"SUBSCRIBE FAILED in transfer error code: %d returned\n",
-			return_code );
+		UpnpPrintf(UPNP_CRITICAL,
+			GENA,
+			__FILE__,
+			__LINE__,
+			"SUBSCRIBE FAILED in transfer error code: %d "
+			"returned\n",
+			return_code);
 		goto error_handler;
 	}
 
-	if(GetHandleInfo(client_handle, &handle_info) != HND_CLIENT) {
+	if (GetHandleInfo(client_handle, &handle_info) != HND_CLIENT) {
 		return_code = GENA_E_BAD_HANDLE;
 		goto error_handler;
 	}
 
 	/* generate client SID */
-	uuid_create(&uid );
-	uuid_unpack(&uid, temp_sid);
-	sprintf(temp_sid2, "uuid:%s", temp_sid);
+	uuid_create(&uid);
+	upnp_uuid_unpack(&uid, temp_sid);
+	rc = snprintf(temp_sid2, sizeof(temp_sid2), "uuid:%s", temp_sid);
+	if (rc < 0 || (unsigned int)rc >= sizeof(temp_sid2)) {
+		return_code = UPNP_E_OUTOF_MEMORY;
+		goto error_handler;
+	}
 	UpnpString_set_String(out_sid, temp_sid2);
 
 	/* create event url */
@@ -556,38 +613,36 @@ int genaSubscribe(
 		return_code = UPNP_E_OUTOF_MEMORY;
 		goto error_handler;
 	}
-	UpnpClientSubscription_set_RenewEventId(newSubscription, -1);
-	UpnpClientSubscription_set_SID(newSubscription, out_sid);
-	UpnpClientSubscription_set_ActualSID(newSubscription, ActualSID);
-	UpnpClientSubscription_set_EventURL(newSubscription, EventURL);
-	UpnpClientSubscription_set_Next(newSubscription, handle_info->ClientSubList);
+	GenlibClientSubscription_set_RenewEventId(newSubscription, -1);
+	GenlibClientSubscription_set_SID(newSubscription, out_sid);
+	GenlibClientSubscription_set_ActualSID(newSubscription, ActualSID);
+	GenlibClientSubscription_set_EventURL(newSubscription, EventURL);
+	GenlibClientSubscription_set_Next(
+		newSubscription, handle_info->ClientSubList);
 	handle_info->ClientSubList = newSubscription;
 
 	/* schedule expiration event */
-	return_code = ScheduleGenaAutoRenew(client_handle, *TimeOut, newSubscription);
+	return_code =
+		ScheduleGenaAutoRenew(client_handle, *TimeOut, newSubscription);
 
 error_handler:
-	if (return_code != UPNP_E_SUCCESS) {
-		UpnpString_delete(ActualSID);
-		UpnpString_delete(EventURL);
-		UpnpClientSubscription_delete(newSubscription);
-	}
+	UpnpString_delete(ActualSID);
+	UpnpString_delete(EventURL);
+	if (return_code != UPNP_E_SUCCESS)
+		GenlibClientSubscription_delete(newSubscription);
 	HandleUnlock();
 	SubscribeUnlock();
 
 	return return_code;
 }
-#endif /* INCLUDE_CLIENT_APIS */
-
+		#endif /* INCLUDE_CLIENT_APIS */
 
 int genaRenewSubscription(
-	UpnpClient_Handle client_handle,
-	const UpnpString *in_sid,
-	int *TimeOut)
+	UpnpClient_Handle client_handle, const UpnpString *in_sid, int *TimeOut)
 {
 	int return_code = GENA_SUCCESS;
-	ClientSubscription *sub = NULL;
-	ClientSubscription *sub_copy = UpnpClientSubscription_new();
+	GenlibClientSubscription *sub = NULL;
+	GenlibClientSubscription *sub_copy = GenlibClientSubscription_new();
 	struct Handle_Info *handle_info;
 	UpnpString *ActualSID = UpnpString_new();
 	ThreadPoolJob tempJob;
@@ -611,25 +666,28 @@ int genaRenewSubscription(
 	}
 
 	/* remove old events */
-	if (TimerThreadRemove(
-		&gTimerThread,
-		UpnpClientSubscription_get_RenewEventId(sub),
-		&tempJob) == 0 ) {
+	if (TimerThreadRemove(&gTimerThread,
+		    GenlibClientSubscription_get_RenewEventId(sub),
+		    &tempJob) == 0) {
 		free_upnp_timeout((upnp_timeout *)tempJob.arg);
 	}
 
-	UpnpPrintf(UPNP_INFO, GENA, __FILE__, __LINE__, "REMOVED AUTO RENEW  EVENT");
+	UpnpPrintf(UPNP_INFO,
+		GENA,
+		__FILE__,
+		__LINE__,
+		"REMOVED AUTO RENEW  EVENT");
 
-	UpnpClientSubscription_set_RenewEventId(sub, -1);
-	UpnpClientSubscription_assign(sub_copy, sub);
+	GenlibClientSubscription_set_RenewEventId(sub, -1);
+	GenlibClientSubscription_assign(sub_copy, sub);
 
 	HandleUnlock();
 
-	return_code = gena_subscribe(
-		UpnpClientSubscription_get_EventURL(sub_copy),
-		TimeOut,
-		UpnpClientSubscription_get_ActualSID(sub_copy),
-		ActualSID);
+	return_code =
+		gena_subscribe(GenlibClientSubscription_get_EventURL(sub_copy),
+			TimeOut,
+			GenlibClientSubscription_get_ActualSID(sub_copy),
+			ActualSID);
 
 	HandleLock();
 
@@ -659,43 +717,39 @@ int genaRenewSubscription(
 	}
 
 	/* store actual sid */
-	UpnpClientSubscription_set_ActualSID(sub, ActualSID);
+	GenlibClientSubscription_set_ActualSID(sub, ActualSID);
 
 	/* start renew subscription timer */
 	return_code = ScheduleGenaAutoRenew(client_handle, *TimeOut, sub);
 	if (return_code != GENA_SUCCESS) {
-		RemoveClientSubClientSID(
-			&handle_info->ClientSubList,
-			UpnpClientSubscription_get_SID(sub));
+		RemoveClientSubClientSID(&handle_info->ClientSubList,
+			GenlibClientSubscription_get_SID(sub));
 	}
 	free_client_subscription(sub_copy);
 	HandleUnlock();
 
 exit_function:
 	UpnpString_delete(ActualSID);
-	UpnpClientSubscription_delete(sub_copy);
+	GenlibClientSubscription_delete(sub_copy);
 	return return_code;
 }
 
-
-void gena_process_notification_event(
-	SOCKINFO *info,
-	http_message_t *event)
+void gena_process_notification_event(SOCKINFO *info, http_message_t *event)
 {
-	struct Upnp_Event event_struct;
+	UpnpEvent *event_struct = UpnpEvent_new();
 	IXML_Document *ChangedVars = NULL;
 	int eventKey;
 	token sid;
-	ClientSubscription *subscription = NULL;
+	GenlibClientSubscription *subscription = NULL;
 	struct Handle_Info *handle_info;
 	void *cookie;
 	Upnp_FunPtr callback;
 	UpnpClient_Handle client_handle;
-	const UpnpString *tmpSID = NULL;
+	UpnpClient_Handle client_handle_start;
+	int err_ret = HTTP_PRECONDITION_FAILED;
 
 	memptr sid_hdr;
-	memptr nt_hdr,
-	nts_hdr;
+	memptr nt_hdr, nts_hdr;
 	memptr seq_hdr;
 
 	/* get SID */
@@ -708,29 +762,30 @@ void gena_process_notification_event(
 
 	/* get event key */
 	if (httpmsg_find_hdr(event, HDR_SEQ, &seq_hdr) == NULL ||
-	    matchstr(seq_hdr.buf, seq_hdr.length, "%d%0", &eventKey) != PARSE_OK) {
-		error_respond( info, HTTP_BAD_REQUEST, event );
+		matchstr(seq_hdr.buf, seq_hdr.length, "%d%0", &eventKey) !=
+			PARSE_OK) {
+		error_respond(info, HTTP_BAD_REQUEST, event);
 		goto exit_function;
 	}
 
 	/* get NT and NTS headers */
 	if (httpmsg_find_hdr(event, HDR_NT, &nt_hdr) == NULL ||
-	    httpmsg_find_hdr(event, HDR_NTS, &nts_hdr) == NULL) {
-		error_respond( info, HTTP_BAD_REQUEST, event );
+		httpmsg_find_hdr(event, HDR_NTS, &nts_hdr) == NULL) {
+		error_respond(info, HTTP_BAD_REQUEST, event);
 		goto exit_function;
 	}
 
 	/* verify NT and NTS headers */
 	if (memptr_cmp(&nt_hdr, "upnp:event") != 0 ||
-	    memptr_cmp(&nts_hdr, "upnp:propchange") != 0) {
+		memptr_cmp(&nts_hdr, "upnp:propchange") != 0) {
 		error_respond(info, HTTP_PRECONDITION_FAILED, event);
 		goto exit_function;
 	}
 
 	/* parse the content (should be XML) */
-	if (!has_xml_content_type(event) ||
-	    event->msg.length == 0 ||
-	    ixmlParseBufferEx(event->entity.buf, &ChangedVars) != IXML_SUCCESS) {
+	if (!has_xml_content_type(event) || event->msg.length == 0 ||
+		ixmlParseBufferEx(event->entity.buf, &ChangedVars) !=
+			IXML_SUCCESS) {
 		error_respond(info, HTTP_BAD_REQUEST, event);
 		goto exit_function;
 	}
@@ -738,78 +793,97 @@ void gena_process_notification_event(
 	HandleLock();
 
 	/* get client info */
-	if (GetClientHandleInfo(&client_handle, &handle_info) != HND_CLIENT) {
+	if (GetClientHandleInfo(&client_handle_start, &handle_info) !=
+		HND_CLIENT) {
 		error_respond(info, HTTP_PRECONDITION_FAILED, event);
 		HandleUnlock();
 		goto exit_function;
 	}
 
-	/* get subscription based on SID */
-	subscription = GetClientSubActualSID(handle_info->ClientSubList, &sid);
-	if (subscription == NULL) {
-		if (eventKey == 0) {
-			/* wait until we've finished processing a subscription  */
-			/*   (if we are in the middle) */
-			/* this is to avoid mistakenly rejecting the first event if we  */
-			/*   receive it before the subscription response */
-			HandleUnlock();
-
-			/* try and get Subscription Lock  */
-			/*   (in case we are in the process of subscribing) */
-			SubscribeLock();
-
-			/* get HandleLock again */
-			HandleLock();
-
-			if (GetClientHandleInfo(&client_handle, &handle_info) != HND_CLIENT) {
-				error_respond(info, HTTP_PRECONDITION_FAILED, event);
-				SubscribeUnlock();
-				HandleUnlock();
-				goto exit_function;
-			}
-
-			subscription = GetClientSubActualSID(handle_info->ClientSubList, &sid);
-			if (subscription == NULL) {
-				error_respond( info, HTTP_PRECONDITION_FAILED, event );
-				SubscribeUnlock();
-				HandleUnlock();
-				goto exit_function;
-			}
-
-			SubscribeUnlock();
-		} else {
-			error_respond( info, HTTP_PRECONDITION_FAILED, event );
-			HandleUnlock();
-			goto exit_function;
-		}
-	}
-
-	/* success */
-	error_respond(info, HTTP_OK, event);
-
-	/* fill event struct */
-	tmpSID = UpnpClientSubscription_get_SID(subscription);
-	strcpy(event_struct.Sid, UpnpString_get_String(tmpSID));
-	event_struct.EventKey = eventKey;
-	event_struct.ChangedVariables = ChangedVars;
-
-	/* copy callback */
-	callback = handle_info->Callback;
-	cookie = handle_info->Cookie;
-
 	HandleUnlock();
 
-	/* make callback with event struct */
-	/* In future, should find a way of mainting */
-	/* that the handle is not unregistered in the middle of a */
-	/* callback */
-	callback(UPNP_EVENT_RECEIVED, &event_struct, cookie);
+	for (client_handle = client_handle_start; client_handle < NUM_HANDLE;
+		client_handle++) {
+		HandleLock();
+
+		/* get client info */
+		if (GetHandleInfo(client_handle, &handle_info) != HND_CLIENT) {
+			HandleUnlock();
+			continue;
+		}
+
+		/* get subscription based on SID */
+		subscription =
+			GetClientSubActualSID(handle_info->ClientSubList, &sid);
+		if (subscription == NULL) {
+			if (eventKey == 0) {
+				/* wait until we've finished processing a
+				 * subscription  */
+				/*   (if we are in the middle) */
+				/* this is to avoid mistakenly rejecting the
+				 * first event if we  */
+				/*   receive it before the subscription response
+				 */
+				HandleUnlock();
+
+				/* try and get Subscription Lock  */
+				/*   (in case we are in the process of
+				 * subscribing) */
+				SubscribeLock();
+
+				/* get HandleLock again */
+				HandleLock();
+
+				if (GetHandleInfo(client_handle,
+					    &handle_info) != HND_CLIENT) {
+					SubscribeUnlock();
+					HandleUnlock();
+					continue;
+				}
+
+				subscription = GetClientSubActualSID(
+					handle_info->ClientSubList, &sid);
+				if (subscription == NULL) {
+					SubscribeUnlock();
+					HandleUnlock();
+					continue;
+				}
+
+				SubscribeUnlock();
+			} else {
+				HandleUnlock();
+				continue;
+			}
+		}
+
+		/* success */
+		err_ret = HTTP_OK;
+
+		/* fill event struct */
+		UpnpEvent_set_EventKey(event_struct, eventKey);
+		UpnpEvent_set_ChangedVariables(event_struct, ChangedVars);
+		UpnpEvent_set_SID(event_struct,
+			GenlibClientSubscription_get_SID(subscription));
+
+		/* copy callback */
+		callback = handle_info->Callback;
+		cookie = handle_info->Cookie;
+
+		HandleUnlock();
+
+		/* make callback with event struct */
+		/* In future, should find a way of mainting */
+		/* that the handle is not unregistered in the middle of a */
+		/* callback */
+		callback(UPNP_EVENT_RECEIVED, event_struct, cookie);
+	}
+
+	error_respond(info, err_ret, event);
 
 exit_function:
 	ixmlDocument_free(ChangedVars);
+	UpnpEvent_delete(event_struct);
 }
 
-
-#endif /* INCLUDE_CLIENT_APIS */
-#endif /* EXCLUDE_GENA */
-
+	#endif /* INCLUDE_CLIENT_APIS */
+#endif         /* EXCLUDE_GENA */
